package net.http.requestfield {
	
	import flash.utils.ByteArray;
	import flash.utils.IDataInput;
	import flash.utils.IDataOutput;
	import net.http.Request;
	
	/**
	 * Класс, представляющий поле в POST-запросе, содержащее файл
	 * 
	 * @author gloomyBrain
	 */
	public final class File implements IByteableField {
		
		private var _name:String;
		private var _data:ByteArray;
		private var _fileName:String;
		private var _mimeType:String;
		
		/**
		 * @param	name		Имя поля в POST-запросе
		 * @param	data		Бинарные данные файла
		 * 
		 * @param	fileName	Имя файла (рекомендуется указывать вместе с расширением)
		 * 						Некоторые сервера по имени файла определяют его тип 
		 * 						и обрабатывают соответствующим образом
		 * @param	mimeType 	Тип отпарвляемого файла. По умолчанию - MIMEType.OCTET_STREAM (бинарные данные)
		 * 
		 */
		public function File(name:String, data:ByteArray, fileName:String = "", mimeType:String = "application/octet-stream") {
			
			_data = data;
			_name = name;
			_fileName = fileName;
			_mimeType = mimeType;
			
		}
		
		/**
		 * Бинарные данные файла
		 */
		public function get data():ByteArray {
			return _data;
		}
		
		/**
		 * Имя поля в запросе
		 */
		public function get name():String {
			return _name;
		}
		
		/**
		 * Имя файла (рекомендуется указывать вместе с расширением)
		 * Некоторые сервера по имени файла определяют его тип 
		 * и обрабатывают соответствующим образом
		 */
		public function get fileName():String {
			return _fileName;
		}
		
		/**
		 * Преобразует исходный экземпляр File в массив байт
		 * Порядок байт устанавливается в BIG_ENDIAN
		 * 
		 * @param	storage	Массив байт, в который дописывается содержимое файла
		 */
		public function toBytes(storage:IDataOutput = null):IDataInput {
			
			if (!storage) storage = new ByteArray();
			
			storage.writeUTFBytes("Content-Disposition: form-data; name=\"" + _name + "\"");
			storage.writeUTFBytes("; filename=\"" + _fileName + "\"");
			storage.writeUTFBytes(Request.LB);
			
			storage.writeUTFBytes("Content-Type: " + _mimeType);
			storage.writeUTFBytes(Request.LB);
			
			storage.writeUTFBytes("Content-Transfer-Encoding: binary");
			storage.writeUTFBytes(Request.LB);
			
			storage.writeUTFBytes(Request.LB);
			
			// file content
			storage.writeBytes(_data, 0, _data.length);
			
			storage.writeUTFBytes(Request.LB);
			
			return storage as IDataInput;
		}
		
	}

}